
import re
import os
from datetime import datetime

import click
import requests
from sh import git

from . import blog
from .helpers import choose_commit_emoji, redirect_output, find_files


@blog.command()
@click.pass_context
def publish(context):
    """Saves changes and sends them to GitHub"""

    branch = get_branch()
    if branch != 'master':
        click.echo("Your current Git branch is '{}' instead of master.".format(
                   branch))
        context.exit(1)

    content_changes = []
    other_changes = []

    content_re = re.compile(r'^content/')
    for path in get_changes():
        if content_re.match(path):
            content_changes.append(path)
        else:
            other_changes.append(path)

    if content_changes:
        click.echo('Changes to be published:\n')
        for path in content_changes:
            click.secho(path, fg='yellow')

        if other_changes:
            click.echo('\nSome changes are not related to content '
                       'of the blog:\n')
            for path in other_changes:
                click.secho(path, fg='blue')
            click.echo('\nYou will have to save those manually.')

        if not click.confirm('\nContinue publishing'):
            click.echo('Aborted!')
            context.exit(1)

        for path in content_changes:
            if os.path.isfile(path) and os.path.splitext(path)[1] == '.md':
                click.echo('Updating modification date: {}'.format(path))
                update_modified(path)

        git.add('content', A=True)
        git.commit(m='Publishing {}'.format(choose_commit_emoji()))
    else:
        click.echo('No changes.')

    click.echo('Pushing to GitHub...')
    git.push('origin', 'master', **redirect_output())

    pr_link = get_pr_link()
    if pr_link:
        click.launch(pr_link)


def update_modified(filename):
    modification_date = datetime.now()
    with click.open_file(filename) as f:
        article_src = f.read()
    article_src = re.sub(
        r'Modified: (.+)',
        'Modified: {:%Y-%m-%d %H:%M}:00'.format(modification_date),
        article_src,
    )
    with click.open_file(filename, 'w') as f:
        f.write(article_src)


def get_changes():
    for line in git('status', porcelain=True):
        match = re.match(r'\s*\S+ (.+)', str(line))
        path = match.group(1)
        yield from find_files(path)


def get_pr_link():
    repo_slug = get_repo_slug()
    repo_url = 'https://api.github.com/repos/{}'.format(repo_slug)

    res = requests.get(repo_url)
    res.raise_for_status()
    repo_info = res.json()

    if not repo_info['fork']:
        return None

    return 'https://github.com/{}/compare/master...{}:master'.format(
        repo_info['parent']['full_name'],
        repo_info['owner']['login']
    )


def get_repo_slug():
    url = str(git.remote('get-url', 'origin'))
    return re.search(r'github\.com[\:\/](.+)\.git$', url).group(1)


def get_branch():
    return re.search(r'\*\s+(\S+)', str(git.branch(no_color=True))).group(1)




import os

import click
from sh import python, pelican

from . import blog
from .helpers import redirect_output


@blog.command()
@click.pass_context
def preview(context):
    """Opens local preview of your blog website"""

    config = context.obj
    state = {'ready': False}

    def open_browser(line):
        if not state['ready'] and line.startswith('Done: Processed'):
            state['ready'] = True
            click.launch('http://localhost:8000')
        return line

    server_cmd = None
    pelican_cmd = None

    os.chdir(config['OUTPUT_DIR'])
    try:
        server_cmd = python(
            '-m', 'http.server', '8000',
            _bg=True,
        )
        pelican_cmd = pelican(
            config['CONTENT_DIR'],
            output=config['OUTPUT_DIR'],
            settings=os.path.join(config['CWD'], config['SETTINGS_PATH']),
            autoreload=True, debug=True, ignore_cache=True,
            _bg=True, **redirect_output(open_browser)
        )
        server_cmd.wait()
        pelican_cmd.wait()

    except:
        if server_cmd is not None:
            server_cmd.process.kill()
        if pelican_cmd is not None:
            pelican_cmd.process.kill()
        raise




import os
import shutil

import click
from sh import git, pelican, ghp_import

from . import blog
from .helpers import redirect_output, choose_commit_emoji


@blog.command()
@click.pass_context
def deploy(context):
    """Uploads new version of the blog website"""

    config = context.obj

    click.echo('Generating HTML...')
    pelican(
        config['CONTENT_DIR'],
        output=config['OUTPUT_DIR'],
        settings=os.path.join(config['CWD'], config['SETTINGS_PATH']),
        _env={'PRODUCTION': '1'},
        **redirect_output()
    )

    click.echo('Removing unnecessary output...')
    unnecessary_paths = [
        'author', 'category', 'drafts', 'tag', 'feeds', 'tags.html',
        'authors.html', 'categories.html', 'archives.html',
    ]
    for path in unnecessary_paths:
        remove_path(os.path.join(config['OUTPUT_DIR'], path))

    if os.environ.get('TRAVIS'):  # Travis CI
        click.echo('Setting up Git...')
        git.config('user.name', git('show', format='%cN', s=True))
        git.config('user.email', git('show', format='%cE', s=True))

        github_token = os.environ.get('GITHUB_TOKEN')
        repo_slug = os.environ.get('TRAVIS_REPO_SLUG')
        origin = 'https://{}@github.com/{}.git'.format(github_token, repo_slug)
        git.remote('set-url', 'origin', origin)

    click.echo('Rewriting gh-pages branch...')
    commit_message = 'Deploying {}'.format(choose_commit_emoji())
    ghp_import('-m', commit_message, config['OUTPUT_DIR'])

    click.echo('Pushing to GitHub...')
    git.push('origin', 'gh-pages:gh-pages', force=True)


def remove_path(path):
    if os.path.isfile(path):
        os.remove(path)
    else:
        shutil.rmtree(path, ignore_errors=True)




import click
from sh import flake8, ErrorReturnCode

from . import blog
from .helpers import redirect_output


@blog.command()
@click.pass_context
def lint(context):
    """Looks for errors in source code of your blog"""

    try:
        flake8('.', exclude='env', **redirect_output())
    except ErrorReturnCode:
        context.exit(1)
